#include <stdio.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <poll.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <time.h>

#include <gtk/gtk.h>

//#include "Fonts.h" // the MetaWatch fonts
#include "oswald.h"
#include "oswald_main.h"

#include "oswald-ui.h"

#define BITMAP_WIDTH	192
#define BITMAP_HEIGHT	192

static oswald_ui *ui_g;

void hal_lcd_set_pixel(gint x, gint y, uint8_t state)
{
	gint ix, iy;

	ix = x*2;
	iy = y*2;

	// we double all pixel to get a bigger picture
	gdk_draw_point(GDK_DRAWABLE(ui_g->pixmap), state ? ui_g->darea->style->black_gc : ui_g->darea->style->white_gc, ix, iy);
	gdk_draw_point(GDK_DRAWABLE(ui_g->pixmap), state ? ui_g->darea->style->black_gc : ui_g->darea->style->white_gc, ix+1, iy);
	gdk_draw_point(GDK_DRAWABLE(ui_g->pixmap), state ? ui_g->darea->style->black_gc : ui_g->darea->style->white_gc, ix, iy+1);
	gdk_draw_point(GDK_DRAWABLE(ui_g->pixmap), state ? ui_g->darea->style->black_gc : ui_g->darea->style->white_gc, ix+1, iy+1);

}

/* updates the actual LCD so that drawing becomes visible */
void hal_lcd_update_display(void)
{
	gtk_widget_queue_draw(ui_g->darea);
}

void hal_lcd_clear_display(void)
{
	gdk_draw_rectangle (ui_g->pixmap,
		ui_g->darea->style->white_gc,
		TRUE,
		0, 0,
		ui_g->darea->allocation.width,
		ui_g->darea->allocation.height);

	gtk_widget_queue_draw(ui_g->darea);
}

static bluetooth_state bt_state = BLUETOOTH_OFF;
static boolean bt_visible = FALSE;

bluetooth_state hal_bluetooth_set_state(bluetooth_state state)
{
	bt_state = state;
	if (bt_state == BLUETOOTH_OFF)
		bt_visible = FALSE;

	return bt_state;
}

bluetooth_state hal_bluetooth_get_state(void)
{
	return bt_state;
}

uint8_t *hal_bluetooth_get_local_bdaddr(void)
{
	static uint8_t local_bdaddr[6] = {0x00, 0x11, 0x22, 0x33, 0x44, 0x55};

	return local_bdaddr;
}

void hal_bluetooth_set_visible(boolean visible)
{
	bt_visible = visible;
}

boolean hal_bluetooth_get_visible(void)
{
	return bt_visible;
}

void hal_bluetooth_send_data(const void *mdat, uint16_t mlen)
{
	g_printerr("write comm %d\n", mlen);
}

const char *hal_get_version_string(void)
{
	return "GTK v0.3";
}

const char *hal_get_buildno_string(void)
{
	return BUILDNO;
}

const char *hal_get_radio_version_string(void)
{
	return "BlueZ";
}

void hal_accelerometer_enable(void)
{
	g_printerr("accel enable\n");
	gtk_widget_set_sensitive(ui_g->x_sc, TRUE);
	gtk_widget_set_sensitive(ui_g->y_sc, TRUE);
	gtk_widget_set_sensitive(ui_g->z_sc, TRUE);
}

void hal_accelerometer_disable(void)
{
	g_printerr("accel disable\n");
	gtk_widget_set_sensitive(ui_g->x_sc, FALSE);
	gtk_widget_set_sensitive(ui_g->y_sc, FALSE);
	gtk_widget_set_sensitive(ui_g->z_sc, FALSE);
}


static gint
configure_event (GtkWidget *widget, GdkEventConfigure *event, gpointer user_data)
{
	oswald_ui *ui = (oswald_ui *)user_data;

	if (ui->pixmap)
		gdk_pixmap_unref(ui->pixmap);

	ui->pixmap = gdk_pixmap_new(widget->window,
		widget->allocation.width,
		widget->allocation.height,
		-1);
	gdk_draw_rectangle (ui->pixmap,
		widget->style->white_gc,
		TRUE,
		0, 0,
		widget->allocation.width,
		widget->allocation.height);

	return TRUE;
}

static gint
expose_event (GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
{
	oswald_ui *ui = (oswald_ui *)user_data;

	gdk_draw_pixmap(widget->window,
		widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		ui->pixmap,
		event->area.x, event->area.y,
		event->area.x, event->area.y,
		event->area.width, event->area.height);

	return FALSE;
}

void button_A_clicked (GtkButton *button, gpointer user_data)
{
	oswald_ui *ui = (oswald_ui *)user_data;

	// g_print("Button-A clicked\n");
	oswald_handle_button_press(BUTTON_A);
}

void button_B_clicked (GtkButton *button, gpointer user_data)
{
	oswald_ui *ui = (oswald_ui *)user_data;

	// g_print("Button-B clicked\n");
	oswald_handle_button_press(BUTTON_B);
}

void button_C_clicked (GtkButton *button, gpointer user_data)
{
	oswald_ui *ui = (oswald_ui *)user_data;

	// g_print("Button-C clicked\n");
	oswald_handle_button_press(BUTTON_C);
}

void button_D_clicked (GtkButton *button, gpointer user_data)
{
	oswald_ui *ui = (oswald_ui *)user_data;

	// g_print("Button-D clicked\n");
	oswald_handle_button_press(BUTTON_D);
}

void button_E_clicked (GtkButton *button, gpointer user_data)
{
	oswald_ui *ui = (oswald_ui *)user_data;

	// g_print("Button-E clicked\n");
	oswald_handle_button_press(BUTTON_E);
}

void button_F_clicked (GtkButton *button, gpointer user_data)
{
	oswald_ui *ui = (oswald_ui *)user_data;

	// g_print("Button-F clicked\n");
	oswald_handle_button_press(BUTTON_F);
}

gboolean button_F_pr (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
	oswald_ui *ui = (oswald_ui *)user_data;
	static gint32 press_time;

	GdkEventButton *bev = (GdkEventButton *)event;
	if (bev->type == GDK_BUTTON_PRESS) {
		press_time = bev->time;
		return FALSE;
	};
	if (bev->type == GDK_BUTTON_RELEASE) {
		if (bev->time > (press_time+1000)) {
			g_print("Button-F long press\n");
			return TRUE;
		};
	};

	return FALSE;
}

void ambientlight_value_changed (GtkRange *range, gpointer  user_data)
{
	oswald_ui *ui = (oswald_ui *)user_data;
	double val;

	val = gtk_range_get_value(range);
	oswald_handle_ambientlight_event((uint8_t) val);
}

void accelX_value_changed (GtkRange *range, gpointer  user_data)
{
	oswald_ui *ui = (oswald_ui *)user_data;
	double val;

	val = gtk_range_get_value(range);
	ui->accel_x = (uint8_t)val;
	oswald_handle_accel_event(ui->accel_x, ui->accel_y, ui->accel_z);
}

void accelY_value_changed (GtkRange *range, gpointer  user_data)
{
	oswald_ui *ui = (oswald_ui *)user_data;
	double val;

	val = gtk_range_get_value(range);
	ui->accel_y = (uint8_t)val;
	oswald_handle_accel_event(ui->accel_x, ui->accel_y, ui->accel_z);
}

void accelZ_value_changed (GtkRange *range, gpointer  user_data)
{
	oswald_ui *ui = (oswald_ui *)user_data;
	double val;

	val = gtk_range_get_value(range);
	ui->accel_z = (uint8_t)val;
	oswald_handle_accel_event(ui->accel_x, ui->accel_y, ui->accel_z);
}

static void create_mainwin(oswald_ui *ui)
{
	GtkWidget *mvb, *hb, *vb, *btn, *sc, *l;

	ui->pixmap = NULL;

	ui->mainwin = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	// gtk_window_set_default_size (GTK_WINDOW (ui->mainwin), 440, 240);
	g_signal_connect(G_OBJECT(ui->mainwin), "destroy", gtk_main_quit, NULL);

	mvb = gtk_vbox_new(FALSE, 5);
	gtk_container_add(GTK_CONTAINER(ui->mainwin), mvb);

	hb = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start (GTK_BOX(mvb), hb, FALSE, FALSE, 5);

	vb = gtk_vbox_new(FALSE, 5);
	gtk_box_pack_start (GTK_BOX(hb), vb, FALSE, FALSE, 5);

	btn = gtk_button_new_with_label(" F ");
	gtk_box_pack_start (GTK_BOX(vb), btn, FALSE, FALSE, 10);
	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(button_F_clicked), ui);
	g_signal_connect(G_OBJECT(btn), "button-press-event", G_CALLBACK(button_F_pr), ui);
	g_signal_connect(G_OBJECT(btn), "button-release-event", G_CALLBACK(button_F_pr), ui);

	btn = gtk_button_new_with_label(" E ");
	gtk_box_pack_start (GTK_BOX(vb), btn, FALSE, FALSE, 10);
	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(button_E_clicked), ui);

	btn = gtk_button_new_with_label(" D ");
	gtk_box_pack_start (GTK_BOX(vb), btn, FALSE, FALSE, 10);
	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(button_D_clicked), ui);

	ui->darea = gtk_drawing_area_new ();
	gtk_box_pack_start (GTK_BOX(hb), GTK_WIDGET(ui->darea), FALSE, FALSE, 5);
	gtk_drawing_area_size (GTK_DRAWING_AREA(ui->darea), BITMAP_WIDTH, BITMAP_HEIGHT);

	gtk_signal_connect (GTK_OBJECT (ui->darea), "expose_event", (GtkSignalFunc) expose_event, ui);
	gtk_signal_connect (GTK_OBJECT (ui->darea), "configure_event", (GtkSignalFunc) configure_event, ui);
	// gtk_signal_connect (GTK_OBJECT (drawing_area), "motion_notify_event", (GtkSignalFunc) motion_notify_event, ui);
	// gtk_signal_connect (GTK_OBJECT (drawing_area), "button_press_event", (GtkSignalFunc) button_press_event, ui);

	gtk_widget_set_events (GTK_WIDGET(ui->darea), GDK_EXPOSURE_MASK
		| GDK_LEAVE_NOTIFY_MASK
		| GDK_BUTTON_PRESS_MASK
		| GDK_POINTER_MOTION_MASK
		| GDK_POINTER_MOTION_HINT_MASK);

	vb = gtk_vbox_new(FALSE, 5);
	gtk_box_pack_start (GTK_BOX(hb), vb, FALSE, FALSE, 5);

	btn = gtk_button_new_with_label(" A ");
	gtk_box_pack_start (GTK_BOX(vb), btn, FALSE, FALSE, 10);
	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(button_A_clicked), ui);

	btn = gtk_button_new_with_label(" B ");
	gtk_box_pack_start (GTK_BOX(vb), btn, FALSE, FALSE, 10);
	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(button_B_clicked), ui);

	btn = gtk_button_new_with_label(" C ");
	gtk_box_pack_start (GTK_BOX(vb), btn, FALSE, FALSE, 10);
	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(button_C_clicked), ui);

	// ambient light sensor
	sc = gtk_vscale_new_with_range (0, 255, 1);
	gtk_box_pack_start (GTK_BOX(hb), sc, FALSE, FALSE, 5);
	g_signal_connect(G_OBJECT(sc), "value-changed", G_CALLBACK(ambientlight_value_changed), ui);

	hb = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start (GTK_BOX(mvb), hb, FALSE, FALSE, 5);

	l = gtk_label_new("X:");
	gtk_box_pack_start (GTK_BOX(hb), l, FALSE, FALSE, 5);
	sc = gtk_hscale_new_with_range (-128, 127, 1);
	gtk_box_pack_start (GTK_BOX(hb), sc, TRUE, TRUE, 5);
	g_signal_connect(G_OBJECT(sc), "value-changed", G_CALLBACK(accelX_value_changed), ui);
	ui->x_sc = sc;
	gtk_widget_set_sensitive(ui->x_sc, FALSE);

	l = gtk_label_new("Y:");
	gtk_box_pack_start (GTK_BOX(hb), l, FALSE, FALSE, 5);
	sc = gtk_hscale_new_with_range (-128, 127, 1);
	gtk_box_pack_start (GTK_BOX(hb), sc, TRUE, TRUE, 5);
	g_signal_connect(G_OBJECT(sc), "value-changed", G_CALLBACK(accelY_value_changed), ui);
	ui->y_sc = sc;
	gtk_widget_set_sensitive(ui->y_sc, FALSE);

	l = gtk_label_new("Z:");
	gtk_box_pack_start (GTK_BOX(hb), l, FALSE, FALSE, 5);
	sc = gtk_hscale_new_with_range (-128, 127, 1);
	gtk_box_pack_start (GTK_BOX(hb), sc, TRUE, TRUE, 5);
	g_signal_connect(G_OBJECT(sc), "value-changed", G_CALLBACK(accelZ_value_changed), ui);
	ui->z_sc = sc;
	gtk_widget_set_sensitive(ui->z_sc, FALSE);

	gtk_widget_show_all(ui->mainwin);
}

static gboolean one_second_tmo_handler (gpointer userdata)
{
	oswald_ui *ui = (oswald_ui *)userdata;

	oswald_one_second_tick();

	return TRUE;
}

static gboolean app_idle_handler (gpointer user_data)
{
	g_print("i");
	if (OswaldState.pending_idle) {
		// call Oswald's idle function
		return TRUE;
	};
	return FALSE;
}

static gboolean centisecond_tmo_handler (gpointer userdata)
{
	oswald_ui *ui = (oswald_ui *)userdata;

	if (ui->centisecond_active)
		oswald_centisecond_tick();
	else
		return FALSE;

	return TRUE;
}

void hal_enable_centisecond_timer(void)
{
	ui_g->centisecond_active = TRUE;
	g_timeout_add(10, centisecond_tmo_handler, ui_g);
}

void hal_disable_centisecond_timer(void)
{
	ui_g->centisecond_active = FALSE;
}

static gboolean halfsecond_tmo_handler (gpointer userdata)
{
	oswald_ui *ui = (oswald_ui *)userdata;

	if (ui->halfsecond_active)
		oswald_halfsecond_tick();
	else
		return FALSE;

	return TRUE;
}

void hal_enable_halfsecond_timer(void)
{
	ui_g->halfsecond_active = TRUE;
	g_timeout_add(500, halfsecond_tmo_handler, ui_g);
}

void hal_disable_halfsecond_timer(void)
{
	ui_g->halfsecond_active = FALSE;
}

void hal_get_rtc(clock_state *rtc)
{
	time_t mt;
	struct tm mtime;

	mt = time(NULL);
	localtime_r(&mt, &mtime);

	rtc->hour = mtime.tm_hour;
	rtc->minute = mtime.tm_min;
	rtc->second = mtime.tm_sec;
	rtc->day = mtime.tm_mday;
	rtc->wday = mtime.tm_wday;
	rtc->month = (mtime.tm_mon + 1);
	rtc->year = (mtime.tm_year + 1900);
	
}

void hal_set_rtc(const clock_state *rtc, boolean set_set)
{
}

void hal_get_power_state(power_state *pwr)
{
	pwr->source = POWER_SOURCE_EXTERNAL;
	pwr->charge_state = POWER_CHARGER_CHARGING;
	pwr->percent = 50;
	pwr->level = 3242; // mV
}

static boolean BacklightState = FALSE;

/* sets the backlight on/off, on=TRUE, off=FALSE */
void hal_lcd_set_backlight(boolean state)
{
	g_print("turn LCD backlight %s\n", state ? "on" : "off");
	BacklightState = state;
}

boolean hal_lcd_get_backlight(void)
{
	return BacklightState;
}

static boolean VibrationState = FALSE;

/* sets the vibration motor on/off, on=TRUE, off=FALSE */
void hal_vibration_set_state(boolean state)
{
	g_print("turn vibration %s\n", state ? "on" : "off");
	VibrationState = state;
}

boolean hal_vibration_get_state(void)
{
	return VibrationState;
}

uint16_t hal_amblight_get_val(void)
{
        return 42;
} 


int main(int argc , char ** argv)
{
	oswald_ui ui;
	time_t mt;
	struct tm mtime;

	ui_g = &ui;

	ui.accel_x = 0;
	ui.accel_y = 0;
	ui.accel_z = 0;
	ui.halfsecond_active = FALSE;
	ui.centisecond_active = FALSE;

	mt = time(NULL);
	localtime_r(&mt, &mtime);

	gtk_init (&argc, &argv);

	create_mainwin(&ui);
	gtk_widget_realize(ui.mainwin);

	oswald_set_time(mtime.tm_hour, mtime.tm_min, mtime.tm_sec, TRUE);
	oswald_set_date(mtime.tm_mday, (mtime.tm_mon + 1), (mtime.tm_year + 1900), TRUE);
	oswald_init();

	g_timeout_add_seconds(1, one_second_tmo_handler, &ui);
	// g_idle_add(app_idle_handler, &ui);

	gtk_main ();
	return 0;
}

